@@ -20,7 +20,7 @@ License along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 """
 
-__version__ = "0.4.1"
+__version__ = "0.4.2"
 
 from libxml2dom.macrolib import *
 from libxml2dom.macrolib import \
@@ -71,8 +71,9 @@ class NamedNodeMap(object):
     methods.
     """
 
-    def __init__(self, node):
+    def __init__(self, node, impl):
         self.node = node
+        self.impl = impl
 
     def getNamedItem(self, name):
         return self.node.getAttributeNode(name)
@@ -196,7 +197,7 @@ class Node(object):
         return NodeList([self.impl.get_node(_node, self) for _node in Node_childNodes(self._node)])
 
     def _attributes(self):
-        return NamedNodeMap(self)
+        return NamedNodeMap(self, self.impl)
 
     def _namespaceURI(self):
         return Node_namespaceURI(self._node)
@@ -458,13 +459,19 @@ class Attribute(Node):
 
 # Document housekeeping mechanisms.
 
-class Document(Node):
+class _Document:
 
-    "A class providing document-level housekeeping."
+    """
+    An abstract class providing document-level housekeeping and distinct
+    functionality.
+    """
 
     def __init__(self, node, impl):
         self._node = node
-        self.impl = impl
+        self.implementation = self.impl = impl
+
+    def _documentElement(self):
+        return self.xpath("*")[0]
 
     def _ownerDocument(self):
         return self
@@ -473,8 +480,18 @@ class Document(Node):
         #print "Freeing document", self._node
         libxml2mod.xmlFreeDoc(self._node)
 
+    documentElement = property(_documentElement)
     ownerDocument = property(_ownerDocument)
 
+class Document(_Document, Node):
+
+    """
+    A generic document class. Specialised document classes should inherit from
+    the _Document class and their own variation of Node.
+    """
+
+    pass
+
 class DocumentType(object):
 
     "A class providing a container for document type information."
